from bioservices import picr



class TestPICR(picr.PICR):

    def __init__(self):
        super(TestPICR, self).__init__(verbose=False)


    def test_getUPIForSequence(self):
        res = self.getUPIForSequence(self._sequence_example, ["IPI", "ENSEMBL", "SWISSPROT"])
        res = self.getUPIForSequence(self._sequence_example, "SWISSPROT", taxid="9606")
        res = self.getUPIForSequence(self._sequence_example, "SWISSPROT", onlyactive=False, includeattributes=False)
        print res

    def test_databases(self):
        assert len(self.databases)>0

    def test_MappedDB(self):
        #res = xmltools.easyXML(res)
        self.getMappedDatabaseNames()

    def test_checkDB(self):
        self._checkDBname("IPI")
        try:
            self._checkDBname("dummy")
            assert False
        except:
            assert True

    def test_getUPIForAccession(self):
        self.getUPIForAccession(self._accession_example, ["SWISSPROT"])
        self.getUPIForAccession(self._accession_example, "SWISSPROT", taxid="9606")
        res = self.getUPIForAccession(self._accession_example, "SWISSPROT", onlyactive=False, includeattributes=False)

    def test_getUPIForBLAST(self):
        self.getUPIForBLAST(self._blastfrag_example, "SWISSPROT", taxid="9606")
        self.getUPIForBLAST(self._blastfrag_example, ["SWISSPROT"], taxid="9606", includeattributes=False)
        self.getUPIForBLAST(self._blastfrag_example, ["SWISSPROT"], taxid="9606", program="blastp",matrix="BLOSUM62")



# -*- python -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2011-2013 - EBI-EMBL
#
#  File author(s): 
#      Thomas Cokelaer <cokelaer@ebi.ac.uk>
#      https://www.assembla.com/spaces/bioservices/team
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://www.assembla.com/spaces/bioservices/wiki
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
#$Id: rhea.py 106 2013-01-30 14:35:05Z leamoerchharder $
"""Interface to the Rhea web services

.. topic:: What is Rhea ?

    :URL: http://www.ebi.ac.uk/rhea/
    :Citations: See http://www.ebi.ac.uk/rhea/about.xhtml

    .. highlights::

        Rhea is a reaction database, where all reaction participants (reactants
        and products) are linked to the ChEBI database (Chemical Entities of 
        Biological Interest) which provides detailed information about structure,
        formula and charge. Rhea provides built-in validations that ensure both 
        elemental and charge balance of the reactions... While the main focus of 
        Rhea is enzyme-catalysed reactions, other biochemical reactions are also 
        are included.

        The database is extensively cross-referenced. Reactions are currently linked 
        to the EC list, KEGG and MetaCyc, and the reactions will be used in the 
        IntEnz database and in all relevant UniProtKB entries. Furthermore, the 
        reactions will also be used in the UniPathway database to generate 
        pathways and metabolic networks.

        -- from Rhea Home page, Dec 2012 (http://www.ebi.ac.uk/rhea/about.xhtml)

"""
from services import RESTService

__all__ = ["Rhea"]



class Rhea(RESTService):
    """Interface to the `Rhea <http://www.ebi.ac.uk/rhea/rest/1.0/>`_ service

    You can search by compound name, ChEBI ID, reaction ID, cross reference
    (e.g., EC number) or citation (author name, title, abstract text, publication ID).
    You can use double quotes - to match an exact phrase - and the following
    wildcards: 

        * ? (question mark = one character),
        * `*` (asterisk = several characters).

    Searching for caffe* will find reactions with participants such as caffeine,
    trans-caffeic acid or caffeoyl-CoA::

        from bioservices import Rhea
        r = Rhea()
        response = r.search("caffe*")

    Searching for a?e?o* will find reactions with participants such as acetoin, 
    acetone or adenosine.::

        from bioservices import Rhea
        r = Rhea()
        response = r.search("a?e?o*")

    See :meth:`search` :meth:`entry` methods for more information about format.

    """
    _url = "http://www.ebi.ac.uk/rhea/rest"
    def __init__(self, version="1.0",  verbose=True):
        """.. rubric:: Rhea constructor

        :param str version: the current version of the interface (1.0)
        :param bool verbose: True by default

        ::

            >>> from bioservices import Rhea
            >>> r = Rhea()
        """
        super(Rhea, self).__init__(name="Rhea", url=Rhea._url, 
            verbose=verbose)
        self.version = version
        self.format_entry = ["cmlreact", "biopax2", "rxn"]

    def search(self, query, format=None):
        """Search for reactions

        :param str query: the search term using format parameter
        :param str format: the biopax2 or cmlreact format (default)


        :Returns: An XML document containing the reactions with undefined
            direction, with links to the corresponding bi-directional ones.
            The format is easyXML.

        ::

            >>> r = Rhea()
            >>> r.search("caffeine")  # id 10280
            >>> r.search("caffeine", format="biopax2")  # id 10280

        The output is in XML format. This page from the Rhea web site explains 
        what are the `data fields <http://www.ebi.ac.uk/rhea/manual.xhtml>`_ of 
        the XML file.

        """
        _format = format    # format is a keyword but we want to use it so rhea
                            # users are not confused.
        if _format == None:
            _format = "cmlreact" # default is cmlreact
        if _format not in ["biopax2", "cmlreact"]:
            raise ValueError("format must be either cmlreact (default) or biopax2")

        url = self.url + "/" + self.version + "/ws/reaction/%s?q=" % _format
        url += query

        response = self.request(url)
        return response


    def entry(self, id, format):
        """Retrieve a concrete reaction for the given id in a given format

        :param int id: the id of a reaction
        :param format: can be rxn, biopax2, or cmlreact
        :Returns: An XML document containing the reactions with undefined
            direction, with links to the corresponding bi-directional ones.
            The format is easyXML.

        ::

            >>> print r.entry(10281, format="rxn")

        The output is in XML format. This page from the Rhea web site explains 
        what are the `data fields <http://www.ebi.ac.uk/rhea/manual.xhtml>`_ of 
        the XML file.
        """
        self.checkParam(format, self.format_entry)
        url = self.url + "/" + self.version + "/ws/reaction/%s/%s" % (format, id)

        if format=="rxn":
            response = self.request(url, format)
        else:
            response = self.request(url)
        return response








#!/usr/bin/python
# -*- coding: latin-1 -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2011-2013 - EMBL-EBI
#
#  File author(s): 
#      https://www.assembla.com/spaces/bioservices/team
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://www.assembla.com/spaces/bioservices/wiki
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
#$Id:$
"""Interface to the PSICQUIC web service

.. topic:: What is PSICQUIC ?

    :URL: http://code.google.com/p/psicquic/
    :REST: http://code.google.com/p/psicquic/wiki/PsicquicSpec_1_3_Rest

    .. highlights::

        "PSICQUIC is an effort from the HUPO Proteomics Standard Initiative
        (HUPO-PSI) to standardise the access to molecular interaction databases
        programmatically. The PSICQUIC View web interface shows that PSICQUIC
        provides access to 25 active service "

        -- Dec 2012


About queries
================

.. rubric:: source: PSICQUIC View web page

To do a search you can use the Molecular Interaction Query Language which is
based on Lucene's syntax.

* Use OR or space ' ' to search for ANY of the terms in a field
* Use AND if you want to search for those interactions where ALL of your terms are found
* Use quotes (") if you look for a specific phrase (group of terms that must
  be searched together) or terms containing special characters that may otherwise
  be interpreted by our query engine (eg. ':' in a GO term)
* Use parenthesis for complex queries (e.g. '(XXX OR YYY) AND ZZZ')
* Wildcards (`*`,?) can be used between letters in a term or at the end of terms to do fuzzy queries,
   but never at the beginning of a term. 
* Optionally, you can prepend a symbol in front of your term.
    *  + (plus): include this term. Equivalent to AND. e.g. +P12345
    *  - (minus): do not include this term. Equivalent to NOT. e.g. -P12345
    *    Nothing in front of the term. Equivalent to OR. e.g. P12345
* Implicit fields are used when no field is specified (simple search). For
  instance, if you put 'P12345' in the simple query box, this will mean the same
  as identifier:P12345 OR pubid:P12345 OR pubauth:P12345 OR species:P12345 OR
  type:P12345 OR detmethod:P12345 OR interaction_id:P12345

=============== =========================================== =============== ======================
Field Name      Searches on                                 Implicit*       Example
=============== =========================================== =============== ======================
idA             Identifier A                                No              idA:P74565
idB             Identifier B                                No              idB:P74565
id              Identifiers (A or B)                        No              id:P74565
alias           Aliases (A or B)                            No              alias:(KHDRBS1 HCK)
identifiers     Identifiers and Aliases undistinctively     Yes             identifier:P74565
pubauth         Publication 1st author(s)                   Yes             pubauth:scott
pubid           Publication Identifier(s) OR                Yes             pubid:(10837477 12029088)
taxidA          Tax ID interactor A: the tax ID or 
                the species name                            No              taxidA:mouse
taxidB          Tax ID interactor B: the tax ID or 
                species name                                No              taxidB:9606
species         Species. Tax ID A or Tax ID B               Yes             species:human
type            Interaction type(s)                         Yes             type:"physical interaction"
detmethod       Interaction Detection method(s)             Yes             detmethod:"two hybrid*"
interaction_id  Interaction identifier(s)                   Yes             interaction_id:EBI-761050
pbioroleA       Biological role A                           Yes             pbioroleA:ancillary
pbioroleB       Biological role B                           Yes             pbioroleB:"MI:0684"
pbiorole        Biological roles (A or B)                   Yes             pbiorole:enzyme
ptypeA          Interactor type A                           Yes             ptypeA:protein
ptypeB          Interactor type B                           Yes             ptypeB:"gene"
ptype           Interactor types (A or B)                   Yes             pbiorole:"small molecule"
pxrefA          Interactor xref A (or Identifier A)         Yes             pxrefA:"GO:0003824"
pxrefB          Interactor xref B (or Identifier B)                         Yes pxrefB:"GO:0003824"
pxref           Interactor xrefs (A or B or Identifier 
                A or Identifier B)                          Yes             pxref:"catalytic activity"
xref            Interaction xrefs (or Interaction 
                identifiers)                                Yes             xref:"nuclear pore"
annot           Interaction annotations and tags            Yes             annot:"internally curated"
udate           Update date                                 Yes             udate:[20100101 TO 20120101]
negative        Negative interaction boolean                Yes             negative:true
complex         Complex expansion                           Yes             complex:"spoke expanded"
ftypeA          Feature type of participant A               Yes             ftypeA:"sufficient to bind"
ftypeB          Feature type of participant B               Yes             ftypeB:mutation
ftype           Feature type of participant A or B          Yes             ftype:"binding site"
pmethodA        Participant identification method A         Yes             pmethodA:"western blot"
pmethodB        Participant identification method B         Yes             pmethodB:"sequence tag identification"
pmethod         Participant identification methods
                 (A or B)                                   Yes             pmethod:immunostaining 
stc             Stoichiometry (A or B). Only true or 
                false, just to be able to filter 
                interaction having stoichiometry available  Yes             stc:true
param           Interaction parameters. Only true or 
                false, just to be able to filter 
                interaction having parameters available     Yes             param:true
=============== =========================================== =============== ======================


"""

from bioservices import RESTService


#http://code.google.com/p/psicquic/wiki/PsicquicSpec_1_3_Rest

#http://www.biocatalogue.org/services/2078#operations

__all__ = ["PSICQUIC"]


class PSICQUIC(RESTService):
    """Interface to the `PSICQUIC <http://code.google.com/p/psicquic/>`_ service

    There are 2 interfaces to the PSICQUIC service (REST and WSDL) but we used
    the REST only.


    This service provides a common interface to more than 25 other services
    related to protein. So, we won't detail all the possiblity of this service.
    Here is an example that consists of looking for interactors of the
    protein ZAP70 within the IntAct database::

        >>> from bioservices import *
        >>> s = PSICQUIC()
        >>> res = s.query("intact", "zap70")
        >>> len(res) # there are 11 interactions found
        11
        >>> # Let us look at the second one in particular:
        >>> for x in res[1].split("\t"): 
        ...     print x
        uniprotkb:O95169
        uniprotkb:P43403
        intact:EBI-716238
        intact:EBI-1211276
        psi-mi:ndub8_human(display_long)|uniprotkb:NADH-ubiquinone oxidoreductase ASHI
        .
        .

    Another reaction with reactome::

        res = s.query("reactome", "Q9Y266")


    .. warning:: PSICQUIC gives access to 25 other services. We cannot create
        a dedicated parsing for all of them. So, the ::`query` method returns
        the raw data. Addition class may provide dedicated parsing in the
        future.

    ... seealso:: :class:`bioservices.biogrid.BioGrid`
    """

    _formats = ["tab25", "tab25", "tab27", "xml25", "count", "biopax", "xgmml",
        "rdf-xml", "rdf-xml-abbrev", "rdf-n3", "rdf-turtle"]

    def __init__(self, verbose=True):
        """.. rubric:: Constructor

        :param bool verbose: print informative messages

        .. doctest:: 

            >>> from bioservices import PSICQUIC
            >>> s = PSICQUIC()

        """
        urlStr = 'http://www.ebi.ac.uk/Tools/webservices/psicquic'
        super(PSICQUIC, self).__init__("PSICQUIC", verbose=verbose, url=urlStr)
        self._registry = None

    def _get_formats(self):
        return PSICQUIC._formats
    formats = property(_get_formats, doc="Returns the possible output formats")

    def read_registry(self):
        """Reads and returns the active registry 

        """
        url = self.url + '/registry/registry?action=ACTIVE&format=txt'
        res = self.request(url, format='txt')
        return res.split()

    def print_status(self):
        """Prints the services that are available

        :return: Nothing

        The output is tabulated. The columns are:

        * names
        * active
        * count
        * version
        * rest URL
        * soap URL
        * rest example
        * restricted

        .. seealso:: If you want the data into lists, see all attributes
            starting with registry such as :meth:`registry_names`
        """
        url = self.url +  '/registry/registry?action=STATUS&format=xml'
        res = self.request(url)
        names = self.registry_names
        counts = self.registry_counts
        versions = self.registry_versions
        actives = self.registry_actives
        resturls = self.registry_resturls
        soapurls = self.registry_soapurls
        restexs = self.registry_restexamples
        restricted = self.registry_restricted
        N = len(names)

        indices = sorted(range(0,N), key=lambda k: names[k])

        for i in range(0,N):
            print("%s\t %s\t %s\t %s\t %s %s %s %s\n" % (names[i], actives[i], 
                counts[i], versions[i], resturls[i], soapurls[i], restexs[i], restricted[i]))


    # todo a property for the version of PISCQUIC

    def _get_registry(self):
        if self._registry == None:
            url = self.url +  '/registry/registry?action=STATUS&format=xml'
            res = self.request(url, format="xml")
            self._registry = res
        return self._registry
    registry = property(_get_registry, doc="returns the registry of psicquic")

    def _get_registry_names(self):
        res = self.registry
        return [x.findAll('name')[0].text for x in res.findAll("service")]
    registry_names = property(_get_registry_names, doc="returns all services available (names)")

    def _get_registry_restricted(self):
        res = self.registry
        return [x.findAll('restricted')[0].text for x in res.findAll("service")]
    registry_restricted = property(_get_registry_restricted, doc="returns restricted status of services" )

    def _get_registry_resturl(self):
        res = self.registry
        data = [x.findAll('resturl')[0].text for x in res.findAll("service")]
        return data
    registry_resturls = property(_get_registry_resturl, doc="returns URL of REST services")

    def _get_registry_restex(self):
        res = self.registry
        data = [x.findAll('restexample')[0].text for x in res.findAll("service")]
        return data
    registry_restexamples = property(_get_registry_restex, doc="retuns REST example for each service")

    def _get_registry_soapurl(self):
        res = self.registry
        return  [x.findAll('soapurl')[0].text for x in res.findAll("service")]
    registry_soapurls = property(_get_registry_soapurl, doc="returns URL of WSDL service")

    def _get_registry_active(self):
        res = self.registry
        return  [x.findAll('active')[0].text for x in res.findAll("service")]
    registry_actives = property(_get_registry_active, doc="returns active state of each service")

    def _get_registry_count(self):
        res = self.registry
        return  [x.findAll('count')[0].text for x in res.findAll("service")]
    registry_counts = property(_get_registry_count, doc="returns number of entries in each service")

    def _get_registry_version(self):
        res = self.registry
        names = [x.findAll('name')[0].text for x in res.findAll("service")]
        N = len(names)
        version = [0] * N
        for i in range(0,N):
            x = res.findAll("service")[i]
            if x.findAll("version"):
                version[i] = x.findAll("version")[0].text
            else:
                version[i] = None 
        return  version
    registry_versions = property(_get_registry_version, doc="returns version of each service")

    def query(self, service, query, output=None, version="current", firstResult=None, maxResults=None):
        """format = count; query = zap70, service intact returns 

        :param str service: a registered service. See :attr:`registry_names`.
        :param str query: a valid query. Can be `*` or a protein name.
        :param str output: a valid format. See s._formats

        ::

            s.query("intact", "brca2", "tab27")
            s.query("intact", "zap70", "xml25")
            s.query("matrixdb", "*", "xml25")

        This is the programmatic approach to this website:

        http://www.ebi.ac.uk/Tools/webservices/psicquic/view/main.xhtml


        Another example consist in accessing the *string* database for fetching 
        protein-protein interaction data of a particular model organism. Here we
        restrict the query to 100 results::

            s.query("string", "species:10090", firstResult=0, maxResults=100, output="tab25")

        """
        params = {}
        if output!=None:
            self.checkParam(output, self.formats)
            params['format'] = output
        else: output="none"

        names = [x.lower() for x in self.registry_names]
        try:
            index = names.index(service)
        except ValueError:
            print("The service you gave (%s) is not registered. See self.registery_names" % service)
            raise ValueError

        # get the base url according to the service requested
        resturl = self.registry_resturls[index]

        if firstResult != None:
            params['firstResult'] = firstResult
        if maxResults != None:
            params['maxResults'] = maxResults

        postData = self.urlencode(params)

        url = resturl  + 'query/' + query 
        if params:
            url += "?" + postData


        if "xml" in output:
            res = self.request(url, format="xml", baseUrl=False)
        else:
            res = self.request(url, format="txt",baseUrl=False)
            res = res.strip().split("\n")

        return res





from bioservices.reactome import Reactome
import unittest


class test_reactome(Reactome):
    def __init__(self):
        super(test_reactome, self).__init__(verbose=False)

    @unittest.skip
    def test_queryPathwaysForReferenceIdentifiers(self):
        self.queryPathwaysForReferenceIdentifiers(["Q9Y266", "P17480", "P20248"])

    def _test_generatePathwayDiagramInSVG(self):
        return self.generatePathwayDiagramInSVG(Id)


    def test_listTopLevelPathways(self):
        self.listTopLevelPathways()







from bioservices.wsdbfetch import * 



class test_WSDbfetch(WSDbfetch):
    def __init__(self):
        super(test_WSDbfetch, self).__init__(verbose=False)

    def test_getSupportedDBs(self):
        res = self.getSupportedDBs()
        res = self.getSupportedDBs()
        assert len(res) >10

    def test_getSupportedFormats(self):
        res = self.getSupportedFormats()
        res = self.getSupportedFormats()
        assert len(res) >10

    def test_getSupportedStyles(self):
        res = self.getSupportedStyles()
        res = self.getSupportedStyles()
        assert len(res) >10

    def test_fetchBatch(self):
        self.fetchBatch("uniprot" ,"wap_mouse", "xml") 

    def test_fetchData(self):
        self.fetchData('uniprot:zap70_human')

    def test_getDatabaseInfo(self):
        res = self.getDatabaseInfo("uniprotkb")
        assert res.displayName == 'UniProtKB'

    def test_getDatabaseInfoList(self):
        assert len(self.getDatabaseInfoList())>10

    def test_getDatavaseInfoList(self):
        self.getDatabaseInfoList()

    def test_getDbFormats(self):
        self.getDbFormats("uniprotkb")

    def test_getFormat(self):
        assert self.getFormatStyles("uniprotkb", "fasta") >= 3
        #['default', 'raw', 'html']

    def test_wrong_db(self):
        try:
            self.getDbFormats("uniprot")
            assert False
        except:
            assert True
        



#!/usr/bin/python
# -*- coding: latin-1 -*-
#
#  This file is part of bioservices software
#
#  Copyright (c) 2011-2013 - EBI-EMBL
#
#  File author(s): 
#      https://www.assembla.com/spaces/bioservices/team
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
#  website: https://www.assembla.com/spaces/bioservices/wiki
#  documentation: http://packages.python.org/bioservices
#
##############################################################################
#$Id: biomodels.py 141 2013-02-06 10:22:25Z cokelaer $
"""This module provides a class :class:`~BioModels` that allows an easy access to all the BioModel service.


.. topic:: What is BioModels ?

    :URL: http://www.ebi.ac.uk/biomodels-main/
    :Service: http://www.ebi.ac.uk/biomodels-main/services/BioModelsWebServices?wsdl
    :Citations: http://www.ncbi.nlm.nih.gov/pubmed/20587024    

    .. highlights::

        "BioModels Database is a repository hosting computational models of biological
        systems. A large number of the provided models are published in the
        peer-reviewed literature and manually curated. This resource allows biologists
        to store, search and retrieve mathematical models. In addition, those models can
        be used to generate sub-models, can be simulated online, and can be converted
        between different representational formats. "

        -- From BioModels website, Dec. 2012


Some keywords used in this module:

======================= ============================================================
identifier              Description/example
======================= ============================================================
ChEBIIds                identifiers of a ChEBI terms (e.g. CHEBI:4991)
id                      model identifier (e.g. BIOMD0000000408 or MODEL1201250000)
modelId                 model identifier (e.g. BIOMD0000000408 or MODEL1201250000)
uniprotIds              Uniprot identifier (e.g., P41000)
taxonomyId              Taxonomy identifier (e.g. 9606)
GOId                    Gene Ontology identifier (e.g. GO:0006915)
publicationIdOrText     publication identifier (PMID or DOI) or text
                        which occurs in the publication's title or abstract
======================= ============================================================

.. testsetup:: biomodels

    from bioservices import *
    s = BioModels()


"""
import copy
import webbrowser
from functools import wraps

from services import WSDLService

__all__ = ["BioModels"]


def checkId(fn):
    """a decorator that checks the validity of a model Id"""
    @wraps(fn)
    def wrapped(self, Id, **kwargs):
        ids = getattr(self, "modelsId")
        if Id in ids:
            res = fn(self, Id, **kwargs)
            return res
        else:
            raise ValueError("""
    Id provided is not a valid ID. See modelsId attribute.""")
    
    return wrapped


class BioModels(WSDLService):
    """Interface to the `BioModels <http://www.ebi.ac.uk/biomodels>`_ service 

    ::

        >>> from bioservices import *
        >>> s = BioModels()
        >>> model = s.getModelSBMLById('BIOMD0000000299')

    The number of models available can be retrieved easily as well as the model IDs::

        >>> len(s)
        >>> s.modelsId

    Most of the BioModels WSDL are available. There are functions added to
    the original interface such as :meth:`extra_getReactomeIds`.


    """
    _url = "http://www.ebi.ac.uk/biomodels-main/services/BioModelsWebServices?wsdl"
    def __init__(self, verbose=True):
        """.. rubric:: Constructor

        :param bool verbose:

        """
        super(BioModels, self).__init__(name="BioModels", url=BioModels._url, verbose=verbose)

	#: used to store all model Ids once for all
        self._modelsId = None

    def __len__(self):
        l = len(self.serv.getAllModelsId())
        return l

    def getAllModelsId(self):
        """Retrieves the identifiers of all published models

        :return: list of model identifiers


        ::

            >>> s.getAllModelsId()

        .. note:: you cal also use the :attr:`modelsId`

        """
        return self.serv.getAllModelsId()

    def _get_models_id(self):
        if self._modelsId == None:
            self._modelsId = self.serv.getAllModelsId()
        return self._modelsId
    modelsId = property(_get_models_id, doc="list of all model Ids")


    def getAllCuratedModelsId(self):
        """Retrieves the identifiers of all published **curated** models

        :return: list of model identifiers

        ::

            >>> s.getAllCuratedModelsId()

        """
        return self.serv.getAllCuratedModelsId()

    def getAllNonCuratedModelsId(self):
        """Retrieves the identifiers of all published **non** **curated** models

        :return: list of model identifiers

        ::

            >>> s.getAllNonCuratedModelsId()

        """
        return self.serv.getAllNonCuratedModelsId()

    @checkId
    def getModelById(self, Id):
        """Retrieves the SBML form of a model (in a string) given its identifier

        :param str Id: a valid ,odel Id

	.. warning:: this method is now deprecated!

        ::

            model = s.getModelById("BIOMD0000000256") 

        Instead, please use: :meth:`getModelSBMLById`

        """
        return self.serv.getModelById(Id)

    @checkId
    def getAuthorsByModelId(self, Id):
        """Retrieves the name of the authors of the publication associated with
        a given model.

        :param str Id: a valid model Id. See :attr:`modelsId` attribute.


        :return: list of names of the publication's authors

	::

            >>> s.getAuthorsByModelId("BIOMD0000000299")
            ['Leloup JC', 'Gonze D', 'Goldbeter A']

        """
        return self.serv.getAuthorsByModelId(Id)

    @checkId
    def getDateLastModifByModelId(self, Id):
        """Retrieves the date of last modification of a given model.

        :param str Id: a valid model Id. See :attr:`modelsId` attribute.

        :return: date of last modification (expressed according to ISO 8601)

        ::

            >>> s.getLastModifiedDateByModelId("BIOMD0000000256")
            '2012-05-16T14:44:17+00:00'


        .. note:: same as :meth:`getLastModifiedDateByModelId`.
        """
        return self.serv.getDateLastModifByModelId(Id)

    @checkId
    def getEncodersByModelId(self, Id):
        """Retrieves the name of the encoders of a given model.

        :param str Id: a valid model Id. See :attr:`modelsId` attribute.

        :return: list of names of the model's encoders


         ::

             >>> s.getEncodersByModelId("BIOMD0000000256")
             ['Lukas Endler']

        """
        return self.serv.getEncodersByModelId(Id)

    @checkId
    def getLastModifiedDateByModelId(self, Id):
        """Retrieves the date of last modification of a given model.

        :param str Id: a valid model Id. See :attr:`modelsId` attribute.

        :return: date of last modification (expressed according to ISO 8601)

        """
        return self.serv.getLastModifiedDateByModelId(Id)

    @checkId
    def getModelNameById(self, Id):
        """Retrieves the name of a model name given its identifier.

        :param str Id: a valid model Id. See :attr:`modelsId` attribute.

        :return: model name

        ::

            >>> print s.getModelNameById("BIOMD0000000256")
            'Rehm2006_Caspase'

        """
        return self.serv.getModelNameById(Id)

    @checkId
    def getModelSBMLById(self, Id):
        """Retrieves the SBML form of a model (in a string) given its identifier

        :param str Id: a valid model Id. See :attr:`modelsId` attribute.

        :return: str - SBML model in a string

        ::

            >>> from bioservices import *
            >>> s = BioModels()
            >>> s.getModelSBMLById('MODEL1006230101')

        """
        return self.serv.getModelSBMLById(Id)

    def getModelsIdByChEBI(self, Id):
        """Retrieves the identifiers of all models which are **associated** to 
        some ChEBI terms. This relies on the method 'getLiteEntity' of the
        ChEBI Web Services (cf. http://www.ebi.ac.uk/chebi/webServices.do).

        :param str Id: a valid model Id. See :attr:`modelsId` attribute.
        :return:  list of model identifiers 

        ::
      
            >>> s.getModelsIdByChEBI("CHEBI:4978")
            ['BIOMD0000000217', 'BIOMD0000000404']

        .. seealso:: :meth:`getSimpleModelsByChEBIIds`, 
            :meth:`getSimpleModelsRelatedWithChEBI`, :meth:`getModelsIdByChEBIId`
        """
        return self.serv.getModelsIdByChEBI(Id)

    def getModelsIdByChEBIId(self, Id):
        """Retrieves the identifiers of all the models which are **annotated** with
        a given ChEBI term.

        :param str Id: identifier of a ChEBI term (e.g. CHEBI:4978)
        :return:  list of model identifiers 

        .. doctest:: biomodels
 
            >>> from bioservices import *
            >>> s = BioModels()
            >>> s.getModelsIdByChEBIId('CHEBI:4978')
            ['BIOMD0000000404']

        .. seealso:: :meth:`getSimpleModelsByChEBIIds`, 
            :meth:`getSimpleModelsRelatedWithChEBI`,
            :meth:`getModelsIdByChEBI` 
        """
        return self.serv.getModelsIdByChEBIId(Id)

    def getSimpleModelsByChEBIIds(self, Ids):
        """Retrieves the models which are annotated with the given ChEBI terms.

        :param str Id: identifier of a ChEBI term (e.g. CHEBI:4978)
 
        :return: list with all models annotated with the provided ChEBI identifiers, 
            as a TreeMap (which uses ChEBI identifiers as keys) 

        .. seealso:: :meth:`getSimpleModelsByChEBIIds`, 
            :meth:`getModelsIdByChEBIId`,
            :meth:`getModelsIdByChEBI` 

        .. warning:: this method returns empty models even with example 
            provided on BioModels website
        """
        return self.serv.getSimpleModelsByChEBIIds(Ids)

    def getSimpleModelsRelatedWithChEBI(self):
        """Retrieves all the models which are annotated with ChEBI terms.

        The output of this function is a lengthy XML document containing 
        utf-8 characters and the models (in simple models format). You can 
        convert it and extract information such as the models ID by using 
        the our xml parser and the following code:: 

            res = s.getSimpleModelsRelatedWithChEBI()
            res = self.easyXML(res.encode('utf-8'))    
            set([x.findall('modelId')[0].text for x in res.getchildren()])

        .. note:: the output is a string. You can convert it to a class that 
            ease its introspection using :meth:`~bioservices.services.Service.easyXML`.
        .. seealso:: :meth:`getSimpleModelsByChEBIIds`, 
            :meth:`getModelsIdByChEBIId`,
            :meth:`getModelsIdByChEBI` 

        """
        return self.serv.getSimpleModelsRelatedWithChEBI()

    @checkId
    def getPublicationByModelId(self, Id):
        """Retrieves the publication's identifier of a given model.

        :param str Id: a valid model Id. See :attr:`modelsId` attribute.

        :return: publication identifier (can be a PMID, DOI or URL)

        ::

            >>> s.getPublicationByModelId("BIOMD0000000256")
            '16932741'

        .. seealso:: You can open the corresponding pubmed HTML page with 
            :meth:`pubmed`
        """
        return self.serv.getPublicationByModelId(Id)

    @checkId
    def getSimpleModelsByIds(self, Id):
        """Retrieves the main information about given models.

        :param str Id: a valid model Id. See :attr:`modelsId` attribute.

        :return: a XML representaton (string) of the model meta information including
            identifier, name, publication identifier date of last modification...

        ::

            >>> model = s.getSimpleModelsByIds("BIOMD0000000256")
           
        .. note:: the output is a string. You can convert it to a class that 
            ease its introspection using :meth:`~bioservices.services.Service.easyXML`.
        """
        res = self.serv.getSimpleModelsByIds(Id)
        return res


    def getModelsIdByPerson(self, personName):
        """Retrieves the identifiers of all models which have a given person as
        author or encoder.


        :param str personName: author's or encoder's name
        :return: list of model identifiers

        ::

             >>> print s.getModelsIdByPerson(u"Novère")

        .. note:: the use of the letter **u** in front of the string to encode special characters.

        """
        return self.serv.getModelsIdByPerson(personName)

    def getSimpleModelsByReactomeIds(self, reacID, raw=False):
        """Retrieves all the models which are annotated with the given Reactome
records.

        :param: list of reactome identifiers (e.g., REACT_1590)
        :param bool raw: return raw data if True

        :return:  models annotated with the provided Reactome identifiers, as a TreeMap (which uses Reactome identifiers as keys) 


        .. seealso:: How to retrieve REACTOME IDs in :meth:`extra_getReactomeIds`
        .. note:: the output is converted to ease introspection 
             using :meth:`~bioservices.services.Service.easyXML`.
        """
        res = self.serv.getSimpleModelsByReactomeIds(reacID)
        if raw!=True:
            res = self.easyXML(res) 
        return res


    def getModelsIdByUniprot(self, text):
        """Retrieves all the models which are associated to the provided UniProt text.

        :param str text: a free UniProt based text
        :return:  list of model identifiers 

        :: 

            >>> s.getModelsIdByUniprot("P10113")
            ['BIOMD0000000033']

        """
        res = self.serv.getModelsIdByUniprot(text)
        return res

    def getModelsIdByUniprotId(self, Id):
        """Retrieves all the models which are annotated with the given UniProt records.

        :param str Id: a valid model Id. See :attr:`modelsId` attribute.
        :return:  list of model identifiers 

        :: 

            >>> s.getModelsIdByUniprot("P10113")
            ['BIOMD0000000033']

    
        """
        res = self.serv.getModelsIdByUniprotId(Id)
        return res

    def getModelsIdByUniprotIds(self, Ids_list):
        """Retrieves all the models which are annotated with the given UniProt records.

        :param str Ids_list: a valid model Id. See :attr:`modelsId` attribute.
        :return:  list of model identifiers 

        :: 

            >>> s.getModelsIdByUniprotIds(["P10113", "P10415"])
            ['BIOMD0000000033', 'BIOMD0000000220']

        """
        return self.serv.getModelsIdByUniprotIds(Ids_list)


    def getModelsIdByName(self, name):
        """Retrieves the models' identifiers which name includes the given keyword.

        :param str name: 
        :return:  array of strings - list of model identifiers

        ::

            >>> res = s.getModelsIdByName("2009")

        """
        
        return self.serv.getModelsIdByName(name)

    def getModelsIdByPublication(self, pubId):
        """Retrieves the identifiers of all models related to one (or more) publication(s).


        :param str pubId: publication identifier PMID or DOI or text which occurs in the publications.
        
        ::

            s.getModelsIdByPublication('18308339')
            ['BIOMD0000000201']

        """
        return self.serv.getModelsIdByPublication(pubId)


    def getModelsIdByGO(self, goId):
        """Retrieves the identifiers of all models related to a Gene Ontology Id.
   
        :param str goId: a free GO text
        :return:  list of model identifiers 

        ::

            >>> s.getModelsIdByGO('0001756')
            ['BIOMD0000000201', 'BIOMD0000000275']

        .. seealso:::meth:`getModelsIdByGOId`

        """
        return self.serv.getModelsIdByGO(goId)

    def getModelsIdByTaxonomy(self, text):
        """Retrieves the models which are associated to the provided Taxonomy text.

        :param str text: free (Taxonomy based) text
        :return:  list of model identifiers 

        """
        return self.serv.getModelsIdByTaxonomy(text)

    def getModelsIdByTaxonomyId(self, taxonomyId):
        """Retrieves the models which are annotated with the given taxon.


        :param str taxonomyId: Taxonomy identifier (e.g. 9606)

        """
        return self.serv.getModelsIdByTaxonomyId(taxonomyId)


    def getSubModelSBML(self, modelId, elementsIDs):
        """Generates the minimal sub-model of a given model in the database including all selected components.

        :param modelId: identifier of the model from which the sub-model will be extracted
        :param elementsIDs: identifiers of the selected elements. Currently only
            supports identifiers from compartments, species, and reactions.

        ::

            s.getSubModelSBML("BIOMD0000000242", "cyclinEdegradation_1")
        """
        return self.serv.getSubModelSBML(modelId, elementsIDs)


    def getModelsIdByGOId(self, GOId):
        """Retrieves the models which are annotated with the given Gene Ontology term.

        :param str GOId: Gene Ontology identifier (e.g. GO:0006915)

        .. seealso:::meth:`getModelsIdByGO`

        :: 

            >>> s.getModelsIdByGOId("GO:0006919")
            ['BIOMD0000000256', 'BIOMD0000000102', 'BIOMD0000000103']

        """
        return self.serv.getModelsIdByGOId(GOId)

    def extra_getChEBIIds(self, start=0, end=100, verbose=False):
        """Retrieve existing chEBI Ids by scanning the models

        :param int start: starting Id
        :param int end: end Id
        :param bool verbose: show the status of the analysis
        :return: list of chEBI Ids that have been found in the DB.

        This method may be useful to know the ChEBI Ids used in models

        For instance, scanning the database for start=0 and end=200, a list of
        191 chEBI Ids are returned and its takes a minute.

        .. doctest:: biomodels

            >>> s.extra_getChEBIIds(80, 84)
            ['CHEBI:81', 'CHEBI:83', 'CHEBI:84']
        """
        if start > end or start<0:
            raise ValueError("start must be positive and lower than end value")
        Ids = []
        for i in range(start, end+1):
            ChEBI = "CHEBI:" +  str(i)
            if i%100 == 0 and i>0:
                if verbose==True:
                    print("%f %% done" % ((i-start)*100./float(end-start)))
            res = self.getModelsIdByChEBI(ChEBI)
            if res:
		Ids.append(ChEBI)
        return Ids

    def extra_getReactomeIds(self, start=0, end=1000, verbose=False):
        """Retrieve REACTOME Ids by scanning the models

        :param int start: starting Id
        :param int end: end Id
        :param bool verbose: show the status of the analysis
        :return: list of reactome IDs that have been found in the DB.

        Search all models for reactome Ids in range 'REACT_start' to 'REACT_end'. 
        Can take a while so do not be greedy and select a short range.

        For instance, scanning the database for start=0 and end=3000, a list of
        106 reactome Id are returned and its takes a minute or two.

        .. doctest:: biomodels

            >>> s.extra_getReactomeIds(0, 100)
            ['REACT_33', 'REACT_42', 'REACT_85', 'REACT_89']
        """
        if start > end or start<0:
            raise ValueError("start must be positive and lower than end value")
        Ids = []
        for i in range(start, end+1):
            if i%100 == 0 and i>0:
                if verbose==True:
                    print("%f %% done" % ((i-start)*100./float(end-start)))
            res = self.serv.getSimpleModelsByReactomeIds(['REACT_%s'%i])
            if 'REACT' in res:
                Ids.append('REACT_%s' % i)
        return Ids


    def extra_getUniprotIds(self, start=10000, end=11000):
        """Retrieve the Uniprot IDs  

        :param int start: starting ID value used to scan the database
        :param int end: ending ID value used to to scan the database
        :return: list of uniprot IDs that have been found in the DB.

        It may be useful to know the uniprot IDs that are available in all
        models. Not all of them are indeed available. This function is slow and
        you should use it with parcimony. This is why we set start and end IDs.

        ::

            >>> s.extra_getUniprotIds(10000,11200)
            ['P10113',
             'P10415',
             'P10523',
             'P10600',
             'P10646',
             'P10686',
             'P10687',
             'P10815',
             'P11071']
        """
        if start > end or start<0:
            raise ValueError("start must be positive and lower than end value")
        self.logging.info("Retrieving the uniprot ID from %s to %s" %(start,end))
        Ids = []
        for i in range(start, end):
            if i%100 == 0 and i>0:
                print("%f  done" % ((i-start)*100./float(end-start)))
            res = self.serv.getSimpleModelsByUniprotIds(['P%s'%i])
            if 'P%s' % i in res:
                Ids.append('P%s' % i)
        return Ids





from bioservices.uniprot import *


class test_UniProt(UniProt):
    def __init__(self):
        super(test_UniProt, self).__init__(verbose=False)

    def test_mapping(self):
        res = self.mapping(fr="ACC", to="KEGG_ID", query='P43403')
        assert res == ['From:ACC', 'To:KEGG_ID', 'P43403', 'hsa:7535']
        try: 
            res = self.mapping(fr="AC", to="KEID", query='P434')
            assert False
        except:
            assert True

    def test_searchUniProtId(self):
        self.searchUniProtId("P09958", format="rdf")
        self.searchUniProtId("P09958", format="xml")
        self.searchUniProtId("P09958", format="txt")
        self.searchUniProtId("P09958", format="fasta")
        self.searchUniProtId("P09958", format="gff")

        try:
            self.searchUniProtId("P09958", format="dummy")
            assert False
        except:
            assert True

    def test_search(self):
        self.search('zap70+AND+organism:9606', format='list')
        self.search("zap70+and+taxonomy:9606", format="tab", limit=3, columns="entry name,length,id, genes")
        self.search("zap70+and+taxonomy:9606", format="tab", limit=3, columns="entry name")

        self.search("ZAP70_HUMAN", format="tab", columns="sequence", limit=1)




from bioservices import *




class test_psicquic(PSICQUIC):
    def __init__(self):
        super(test_psicquic, self).__init__(verbose=False)

    def test_read_registry(self):
        self.read_registry()

    def test_print_status(self):
        self.print_status()

    def test_registry(self):
        N1 = len(self.registry_names)
        N2 = len(self.registry_versions)
        N2 = len(self.registry_restricted)
        assert N1 == N2

    def test_query(self):

        self.query("intact", "brca2", "tab27")
        self.query("intact", "zap70", "xml25")
        self.query("matrixdb", "*", "xml25")
        try:
            self.query("matxdb", "*", "xml25")
            assert False
        except:
            assert True


        self.query("matrixdb", "*", "xml25", firstResult=10, maxResults=10)

        # accessing the string DB
        self.query("string", "species:10090", firstResult=0, maxResults=100, output="tab25")

